from .decorators import *


def auto_doc(dict_functions, app_name, options={}):
    to_decorate_functions = ["before_all", "after_all", "before_feature",
                             "after_feature", "before_scenario", "after_scenario",
                             "before_step", "after_step"]
    for func in to_decorate_functions:
        decorator = eval(decorator_for_function(func))
        if func in dict_functions:
            dict_functions[func] = decorator(dict_functions[func], options, app_name)
        else:
            fake_function = lambda *args: None
            fake_function.__globals__["__file__"] = dict_functions["__file__"]
            dict_functions[func] = decorator(fake_function, options, app_name)


def decorator_for_function(function_name):
    words = function_name.split('_')
    return "".join([w.capitalize() for w in words]) + 'Decorator'





from pkg_resources import resource_string
from jinja2 import Template as JinjaTemplate
from string import Template
import os
from docme.output_formats import OutputFormat

class HtmlDocumentation(object):

    def __init__(self, options):
        self.user_options = options
        self.string = """
            <html>
                <head>
                    <meta charset="utf-8"/>
                    <script src="https://code.jquery.com/jquery-3.4.1.slim.min.js" integrity="sha384-J6qa4849blE2+poT4WnyKhv5vZF5SrPo0iEjwBvKU7imGFAV0wwj1yYfoRSJoZ+n" crossorigin="anonymous"></script>
<script src="https://cdn.jsdelivr.net/npm/popper.js@1.16.0/dist/umd/popper.min.js" integrity="sha384-Q6E9RHvbIyZFJoft+2mJbHaEWldlvI9IOYy5n3zV9zzTtmI3UksdQRVvoxMfooAo" crossorigin="anonymous"></script>
<script src="https://stackpath.bootstrapcdn.com/bootstrap/4.4.1/js/bootstrap.min.js" integrity="sha384-wfSDF2E50Y2D1uUdj0O3uMBJnjuUD4Ih7YwaYd1iqfktj0Uod8GCExl3Og8ifwB6" crossorigin="anonymous"></script>
                    <link rel="stylesheet" href="https://stackpath.bootstrapcdn.com/bootstrap/4.4.1/css/bootstrap.min.css" integrity="sha384-Vkoo8x4CGsO3+Hhxv8T/Q5PaXtkKtu6ug5TOeNV6gBiFeWPGFN9MuhOf23Q9Ifjh" crossorigin="anonymous">
        """
        self.string += "css_path" in options and "<link rel = 'stylesheet' type=text/css href = '{}'  >".format(options["css_path"]) or " "
        self.string += """        
                </head>
                <body>
                <div class="container">
        """
    
    def ln(self):
        self.string += "<br/>"

    def hr(self):
        self.string += "<hr/>"

    def add_feature(self, feature):
        if "feature_html" in self.user_options:
            feature_string = open(
                self.user_options['feature_html'], "r").read()
        else:
            feature_string = resource_string(
                "docme", "assets/feature.html").decode('utf-8')
        t = Template(feature_string)
        self.string += t.substitute(
            feature_title=feature.name,
            feature_description=((feature.text_description and feature.text_description) or ' ')
        )
 
    def add_scenario(self, scenario):
        if "scenario_html" in self.user_options:
            scenario_string = open(
                self.user_options['scenario_html'], "r").read()
        else:
            scenario_string = resource_string(
                "docme", "assets/scenario.html").decode('utf-8')
        t = Template(scenario_string)
        self.string += t.substitute(
            scenario_title=scenario.name,
            scenario_description=((scenario.text_description and scenario.text_description) or ' ')
        )

    def add_examples_list(self, context):
        if context.table:
            self.string += "<ul>"
            for dado in context.table:
                self.string += """
                                <li>
                                    Preencha o campo {} com {}
                                </li>
                                """.format(dado['campo'], dado['valor'])
            self.string += "</ul>"

    def new_page(self):
        if not(self.string.endswith("<p class=new_page></p>")):
            self.string += "<p class=new_page></p>"

    def add_step(self, image_path, step, context):
        if step.vertical:
            if "step_vertical_html" in self.user_options:
                step_string = open(self.user_options['step_vertical_html'], "r").read()
            else:
                step_string = resource_string(
                "docme", "assets/step_vertical.html").decode('utf-8')
        else:
            if "step_horizontal_html" in self.user_options:
                step_string = open(
                    self.user_options['step_horizontal_html'], "r").read()
            else:
                step_string = resource_string(
                    "docme", "assets/step_horizontal.html").decode('utf-8')
        t = JinjaTemplate(step_string)
        self.string += t.render(step_screenshot=image_path,
                                step=step,
                               )
        if step.break_page:
            self.new_page()

    def save(self, docs_dir):
        if "css_file" in self.user_options:
            css_string = open(self.user_options['css_file'], "r").read()
        else:
            css_string = resource_string(
                "docme", "assets/docme.css").decode('utf-8')
        self.string += """
                </body>
            </html>
            <style>
                @media print {
                    .row.step {
                        page-break-before: auto; /* 'always,' 'avoid,' 'left,' 'inherit,' or 'right' */
                        page-break-after: auto; /* 'always,' 'avoid,' 'left,' 'inherit,' or 'right' */
                        page-break-inside: avoid; /* or 'auto' */
                    }
                }
             """ + css_string + "</style>"
        output_formats = self.user_options["output-formats"] if "output-formats" in\
                             self.user_options else ['pdf']
        OutputFormat.save(self.string, output_formats, docs_dir)
           



from pkg_resources import resource_string
import json
import os
from docme.output_formats import OutputFormat


class JsonDocumentation(object):

    def __init__(self, options, app_name):
        self.user_options = options
        self.json = {app_name: {}}
        self.app_name = app_name
        
    def add_feature(self, feature):
        self.json[self.app_name][feature.name] = {
            'description': feature.text_description and feature.text_description,
            'scenarios': {},
        } 
        self.atual_feature = feature.name

    def add_scenario(self, scenario, fixtures):
        self.json[self.app_name][self.atual_feature]['scenarios'][scenario.name] = {
            'description': (scenario.text_description and scenario.text_description) or '',
            'fixtures': fixtures,
            'steps': {}
        }
        self.atual_scenario = scenario.name


    def add_step(self, current_path, step):
        if self.json[self.app_name][self.atual_feature]['scenarios'][self.atual_scenario]['steps'] == {}:
            self.json[self.app_name][self.atual_feature]['scenarios'][self.atual_scenario]['first_path'] = current_path
            # self.json[self.app_name][self.atual_feature]['scenarios'][self.atual_scenario]['dumpdata_path'] = step.dumpdata_path
        if current_path in self.json[self.app_name][self.atual_feature]['scenarios'][self.atual_scenario]['steps']:
            self.json[self.app_name][self.atual_feature]['scenarios'][self.atual_scenario]['steps'][current_path].append(
                {
                    "title": step.title,
                    "description": step.text or '',
                }
            )
        else:
            self.json[self.app_name][self.atual_feature]['scenarios'][self.atual_scenario]['steps'][current_path] = [{
                    "title": step.title,
                    "description": step.text or '',
                }]
        self.json[self.app_name][self.atual_feature]['scenarios'][self.atual_scenario][
            'steps'][current_path][-1]['examples'] = self.examples_list(step)

    def examples_list(self, step):
        if step.table and step.with_form_example:
            _string = "<ul>"
            for dados in step.table:
                _string += """
                                <li>
                                    Preencha o campo {} com {}
                                </li>
                                """.format(dados[step.table_label], dados[step.table_value])
            _string += "</ul>"
            return _string
        elif step.table and step.with_data_example:
            _string = "<ul>"
            for dados in step.table:
                _string += """
                                <li>
                                    {}
                                </li>
                                """.format(dados.cells[0])
            _string += "</ul>"
            return _string
        else:
            return ""

    def save(self, docs_dir):
        json_path = os.path.join(docs_dir, "doc.json")
        with open(json_path, 'w', encoding='utf8') as jp:
            json.dump(self.json, jp, indent=2)



import pdfkit
import os


class OutputFormat(object):

    def __init__(self, html_string, docs_dir):
        self.html_string = html_string
        self.docs_dir = docs_dir

    @classmethod
    def save(cls, html_string, output_formats, docs_dir):
        class_formats = {
            'pdf': PdfFormat,
            'html': HtmlFormat
        }
        for output_format in output_formats:
            output_format in class_formats and\
                class_formats[output_format](html_string, docs_dir).save()

class PdfFormat(OutputFormat):

    def save(self):
        options = {
            'page-size': 'Letter',
            'margin-top': '0.5in',
            'margin-right': '0.75in',
            'margin-bottom': '0.5in',
            'margin-left': '0.75in',
            'encoding': "UTF-8",
            'footer-left': "This is a footer",
            'footer-line': '',
            'footer-font-size': '7',
            '--footer-right': 'right footer',

            'custom-header': [
                ('Accept-Encoding', 'gzip')
            ],
            'no-outline': None
        }
        path = os.path.join(self.docs_dir, "doc.pdf")
        config = pdfkit.configuration(wkhtmltopdf='/usr/bin/wkhtmltopdf')
        pdfkit.from_string(self.html_string,
                           path, configuration=config,
                           options=options)


class HtmlFormat(OutputFormat):

    def save(self):
        html_path = os.path.join(self.docs_dir, "doc.html")
        html_doc = open(html_path, "w+")
        html_doc.writelines(self.html_string)
        html_doc.close()



import os
import sys
import shutil
from django.conf import settings
from urllib.parse import urlparse
from django.core.management import call_command
from docme.html_documentation import HtmlDocumentation
from docme.json_documentation import JsonDocumentation
from functools import reduce
import inspect
import types

class EnvironmentFunctionDecorator(object):

    def __init__(self, func, options, app_name):
        self.function = func
        self.options = options
        self.app_name = app_name

    def docs_dir(self):
        return os.path.join(os.path.dirname(self.function.__globals__["__file__"]), 'docs')

    def _clear_name(self, text):
        return reduce(lambda t, param: t.replace(param, ""), [text, ":before", ":docme",
                                                              ":dumpme", ":notitle", ":vertical",
                                                              ":withformexample", ":autotour", ":withdataexample"])

    def text_description(self, element):
        return "<br/>".join(element.description)

class BeforeAllDecorator(EnvironmentFunctionDecorator):

    def __call__(self, context):
        shutil.rmtree(self.docs_dir(), ignore_errors=True)
        setattr(context, 'html_documentation', self.create_html_doc())
        if 'json' in self.options['output-formats']:
            setattr(context, 'json_documentation', self.create_json_doc())
        for i, feature in enumerate(context._runner.features):
            feature.docme = ":docme" in feature.name
            feature.auto_tour = ":autotour" in feature.name
            setattr(feature, 'name', self._clear_name(feature.name))
            setattr(feature, 'index', i)
            setattr(feature, 'text_description', self.text_description(feature))
        self.function(context)

    def create_html_doc(self):
        os.makedirs(self.docs_dir(), exist_ok=True)
        return HtmlDocumentation(self.options)

    def create_json_doc(self):
        os.makedirs(self.docs_dir(), exist_ok=True)
        return JsonDocumentation(self.options, self.app_name)

class AfterAllDecorator(EnvironmentFunctionDecorator):

    def __call__(self, context):
        context.json_documentation.save(self.docs_dir())
        context.html_documentation.save(self.docs_dir())
        self.function(context)


class BeforeFeatureDecorator(EnvironmentFunctionDecorator):

    def __call__(self, context, feature):
        if feature.docme:
            context.html_documentation.add_feature(feature)
        if feature.auto_tour:
            context.json_documentation.add_feature(feature)
        for i, scenario in enumerate(feature.scenarios):
            setattr(scenario, "dumpme", ":dumpme" in scenario.name)
            setattr(scenario, "before", ":before" in scenario.name)
            setattr(scenario, "docme", ":docme" in scenario.name)
            setattr(scenario, "auto_tour", ":autotour" in scenario.name)
            setattr(scenario, "name", self._clear_name(scenario.name))
            setattr(scenario, "index", i)
            setattr(scenario, 'text_description',
                    self.text_description(scenario))

        self.function(context, feature)


class AfterFeatureDecorator(EnvironmentFunctionDecorator):

    def __call__(self, context, feature):
        if feature.docme:
            context.html_documentation.new_page()
        self.function(context, feature)


class BeforeScenarioDecorator(EnvironmentFunctionDecorator):

    def __call__(self, context, scenario):
        if scenario.auto_tour:
            context.json_documentation.add_scenario(scenario, context.fixtures)
            for i, step in enumerate(scenario.steps):
                setattr(step, 'auto_tour', ':autotour' in step.name)
                if i == 0:
                    setattr(step, 'dumpdata', True)
        if scenario.docme:
            context.html_documentation.ln()
            context.html_documentation.add_scenario(scenario)
            context.html_documentation.hr()
            for i, step in enumerate(scenario.steps):
                setattr(step, 'documented_step', False)
                setattr(step, 'scenario', scenario)
                setattr(step, 'no_title', ':notitle' in step.name)
                if ':docme' in step.name:
                    setattr(step, 'vertical', ':vertical' in step.name)
                    setattr(step, 'documented_step', True)
                    setattr(step, 'with_form_example', ":withformexample" in step.name)
                    setattr(step, 'with_data_example', ":withdataexample" in step.name)
                    setattr(step, 'break_page', i % 2 == 0)
                if i == 0:
                    setattr(step, 'dump_before', scenario.dumpme and scenario.before)
                setattr(step, 'dump_dir',
                        self.dump_dir(scenario))
                setattr(step, 'name', self._clear_name(step.name))
                step.title = "" if step.no_title else step.name.capitalize()


        self.function(context, scenario)

    def dump_dir(self, scenario):
        return os.path.join(self.docs_dir(),
                            scenario.feature.name,
                            scenario.name.replace(':dumpme', '')).replace(' ', '_')


class AfterScenarioDecorator(EnvironmentFunctionDecorator):

    def __call__(self, context, scenario):
        if scenario.dumpme and not(scenario.before):
            os.makedirs(self.dump_dir(scenario), exist_ok=True)
            command = 'pg_dump -U {} -d {} > {}'.format(
                settings.DATABASES['default']['USER'],
                settings.DATABASES['default']['NAME'],
                self.dump_path(scenario).replace(' ', '\ '))
            os.system(command)
        if scenario.docme:
            context.html_documentation.new_page()

        self.function(context, scenario)

    def dump_path(self, scenario):
        return os.path.join(self.dump_dir(scenario), 'dump.sql')

    def dump_dir(self, scenario):
        return os.path.join(self.docs_dir(),
                            scenario.feature.name,
                            scenario.name)


class BeforeStepDecorator(EnvironmentFunctionDecorator):

    def __call__(self, context, step):
        if hasattr(step, "dump_before") and step.dump_before:
            os.makedirs(step.dump_dir, exist_ok=True)
            command = 'pg_dump -U {} -d {} > {}'.format(
                settings.DATABASES['default']['USER'],
                settings.DATABASES['default']['NAME'],
                self.dump_path(step).replace(' ', '\ '))
            os.system(command)
        # if hasattr(step, "dumpdat a") and step.dumpdata:
        #     os.makedirs(self.dumpdata_dir(), exist_ok=True)
        #     with open(self.dumpdata_path(step), 'w+') as f:
        #         call_command('dumpdata', stdout=f)
        #         setattr(step, 'dumpdata_path', self.dumpdata_path(step))
        if hasattr(step, "documented_step") and step.documented_step and step.with_form_example:
            for i, h in enumerate(step.table.headings):
                if ':label' in h:
                    step.table_label = h.replace(':label', '')
                    step.table.__dict__['headings'][i] = h.replace(':label', '')
                if ':value' in h:
                    step.table_value = h.replace(':value', '')
                    step.table.__dict__['headings'][i] = h.replace(':value', '')
        if hasattr(step, "documented_step") and step.documented_step:
            os.makedirs(self.image_path(step), exist_ok=True)
            image_path = context.browser.screenshot(
                self.image_path(step), full=True)
            # setattr(step, 'capitalized_name', step.name.capitalize())
            context.html_documentation.add_step(image_path, step, context)
            setattr(step, "documented_step", False)
        if hasattr(step, "auto_tour") and step.auto_tour:
            if urlparse(context.browser.url).query:
                path = urlparse(context.browser.url).path + "/?" + urlparse(context.browser.url).query
            else:
                path = urlparse(context.browser.url).path
            context.json_documentation.add_step(path.replace('//', '/'), step)
        self.function(context, step)

    def image_path(self, step):
        return os.path.join(step.dump_dir, 'images/')
    def dump_path(self, step):
        return os.path.join(step.dump_dir, 'dump_before.sql')
    
    def dumpdata_path(self, step):
        return os.path.join(self.dumpdata_dir(), "{}_{}.json".format(step.scenario.feature.index, step.scenario.index))
    
    def dumpdata_dir(self):
        return os.path.join(self.docs_dir(), ".dumpsdatas")

class AfterStepDecorator(EnvironmentFunctionDecorator):

    def __call__(self, context, step):
        if hasattr(step, "documented_step") and step.documented_step:
            context.html_documentation.ln()
            context.html_documentation.ln()
        self.function(context, step)



from django.conf import settings
from django.core.management import call_command
from pkg_resources import resource_string
from django.shortcuts import render
from django.http import HttpResponse, JsonResponse
from django.template import RequestContext, loader
from jinja2 import Template
import json

def json_script():
    json_scripts = [json.load(open(json_script))
                    for json_script in settings.AUTO_TOUR_SCRIPTS]
    json_doc = {}
    for json_script in json_scripts:
        json_doc.update(json_script)
    return json_doc

def auto_tour_index(request):
    return render(request, 'docme/index.html', {'json_doc':json_script()})

def start_tour(request):
    scenario = json_script()[request.GET['app']][request.GET['feature_name']
                                                 ]['scenarios'][request.GET['scenario_name']]
    call_command('flush')
    # call_command('sync_permissions', verbosity=0)
    for fixture in scenario['fixtures']:
        call_command('loaddata', fixture, skip_checks=True)
    # ==== Lixo ====
    from comum.models import User
    from ponto.models import Frequencia, Maquina, Observacao, ObservacaoChefia
    from django.contrib.auth.models import Group
    from rh.models import Servidor
    from datetime import datetime, date
    us = User.objects.get(username='1111111')
    us.set_password('T3st3!@#')
    us.save()
    usuario = User.objects.get(username='1111111')
    grupo = Group.objects.get(name='Servidor')
    usuario.groups.add(grupo)
    usuario = User.objects.get(username='1111111')
    grupo = Group.objects.get(name='Chefe de Setor')
    usuario.groups.add(grupo)
    servidor = Servidor.objects.get(matricula='3333333')
    maquina = Maquina.objects.get(id=55)
    Frequencia(vinculo=servidor.vinculo_set.all()[0], horario=datetime(
        2018, 1, 21, 7, 0, 1), acao='E', maquina=maquina).save()
    Frequencia(vinculo=servidor.vinculo_set.all()[0], horario=datetime(
        2018, 1, 21, 12, 0, 1), acao='S', maquina=maquina).save()
    Frequencia(vinculo=servidor.vinculo_set.all()[0], horario=datetime(
        2018, 1, 21, 13, 0, 1), acao='E', maquina=maquina).save()
    Frequencia(vinculo=servidor.vinculo_set.all()[0], horario=datetime(
        2018, 1, 21, 16, 00, 1), acao='S', maquina=maquina).save()
    return JsonResponse({'next_path': scenario['first_path']})


def get_steps(request):
    scenario = json_script()[request.GET['app']][request.GET['feature_name']
                                                 ]['scenarios'][request.GET['scenario_name']]
    return JsonResponse({'steps': scenario['steps'][request.GET['current_path']]})
